#include "Decomp3D.h"

//------------------------------------------------------------------------------

Decomp3D::Decomp3D() {

  fluidDom = new FluidDomain3D;

  numSub = 0;
  connectedDomain = NULL;
  nConnect   = NULL;
  subToElem  = NULL;
  subToNode  = NULL;
  subToFace  = NULL;
  nodeToSub  = NULL;
  subToSub   = NULL;
  interfNode = NULL;
  

}

//------------------------------------------------------------------------------

Decomp3D::~Decomp3D() {

  delete fluidDom;

  if (connectedDomain)
    delete [] connectedDomain[0];

  if (interfNode)
    for (int i=0; i<numSub; i++)
      delete interfNode[i];

  delete [] connectedDomain;
  delete [] nConnect;
  delete [] subToElem;
  delete [] subToNode;
  delete [] subToFace;
  delete [] nodeToSub;
  delete [] subToSub;
  delete [] interfNode;

}


//------------------------------------------------------------------------------

void Decomp3D::getFluidDomain(char* name) {

  fluidDom->getGeometry(name);

}

//------------------------------------------------------------------------------
	
void Decomp3D::getDecomp(char* name) {

  int *connect = new int[fluidDom->elems.size()];
  char buffer[80];
  int i;	
  int nele;

  if(name!=NULL){
    ifstream decompFilePtr(name,ios::in);
    if(!decompFilePtr){
      cerr<<"Error opening file: "<<name<<endl;
      cerr<<"DONE"<<endl;
      decompFilePtr.close();
      exit(1);
    }

    // The input file is read a line at a time into a buffer and
    // scanned for the word Decomposition in the first 13 characters.
    // Once the the key word is found the routine begins reading in
    // the decompostion file info and subToElem is set.
    decompFilePtr.get(buffer[0]);
    i=0;
    while(buffer[i]!='\n'){
      i++;
      decompFilePtr.get(buffer[i]);
    }
    while (strncmp(buffer,"Decomposition",13)){
      decompFilePtr.get(buffer[0]);
      i=0;
      while(buffer[i]!='\n'){
	i++;
	decompFilePtr.get(buffer[i]);
      }
    }
    
    decompFilePtr>>numSub;
    if(99999<numSub||numSub<0){
      cerr<<"The number of subdomains must be positive and can't "; 
      cerr<<"exceed 99999."<<endl;
      cerr<<"DONE"<<endl;
      exit(1);
    }
    int *cx = new int[numSub+1];
    
    int curEle = 0;
    for(int isub=0; isub < numSub; ++isub) {
      decompFilePtr>>nele;
      cx[isub] = curEle;
      if(curEle + nele > fluidDom->elems.size()) {
	cerr<<"This decomposition contains more elements than the ";
	cerr<<"original mesh."<<endl;
	cerr<<"DONE"<<endl;
	exit(1);
      }
      for(int iele=0; iele < nele; ++iele) {
	decompFilePtr>>connect[curEle];
	connect[curEle++]--;
      }
    }
    cx[numSub] = curEle;
    subToElem = new Connectivity(numSub,cx,connect);
  }
  else{
    numSub=1;
    int* cx=new int[2];

    cx[0]=0;
    cx[1]=fluidDom->elems.size();
    for(i=0;i<fluidDom->elems.size();i++)
      connect[i]=i;

    subToElem = new Connectivity(numSub,cx,connect);
  }
}

//------------------------------------------------------------------------------

void Decomp3D::getDomainConnect() {

  // create each subdomain's intertri lists
  int iSub, jSub, subJ, iNode;
  int totConnect;

  if(numSub>1){
    int *flag = new int[numSub];
    nConnect  = new int[numSub];

    for(iSub=0; iSub < numSub; ++iSub) {
      flag[iSub] = -1;
      nConnect[iSub] = 0;
    }
    
    // Count connectivity
    totConnect = 0;
    for(iSub=0; iSub < numSub; ++iSub) {

      // loop over the nodes connected to the current subdomain
      for(iNode = 0; iNode < subToNode->num(iSub); ++iNode) { 
      	int thisNode = (*subToNode)[iSub][iNode];

	// loop over the subdomains connected to this Node
      	for(jSub = 0; jSub < nodeToSub->num(thisNode); ++jSub) 
	  if( (subJ=(*nodeToSub)[thisNode][jSub]) > iSub) {
	    // we only deal with connection to highered numbered 
	    // subdomains so that we guarantee symmetry of lists

	    if(flag[subJ] != iSub) {
	      flag[subJ] = iSub;
	      nConnect[iSub]++;
	      nConnect[subJ]++;
	      totConnect += 2;
	    }
	  }
      }
    }

    int	**nodeCount  = new int*[numSub];
    int **remoteID   = new int*[numSub];
    int *whichLocal  = new int[numSub];
    int *whichRemote = new int[numSub];
    
    // This pointer to as list of pointers
    // is used to construct a 2D array 
    // containing the subdomains that touch
    // a particular subdomain.  Both nConnect
    // and connectedDoamin are necessary to
    // maintain the ordering used in interfNode
    connectedDomain    = new int*[numSub];
    connectedDomain[0] = new int[2*totConnect];
    
    // Set pointers
    int *allAlloc = connectedDomain[0];
    for(iSub=0; iSub < numSub; ++iSub) {
      connectedDomain[iSub] = allAlloc;
      allAlloc += nConnect[iSub];

      remoteID[iSub] = allAlloc;
      allAlloc += nConnect[iSub];
    }
    
    for(iSub=0; iSub < numSub; ++iSub) {
      nodeCount[iSub] = new int[nConnect[iSub]];

      // Reset flag and nConnect for subdomain iSub
      flag[iSub] = -1;
      nConnect[iSub] = 0;      
    }
        
    for(iSub=0; iSub < numSub; ++iSub) {
      for(iNode = 0; iNode < subToNode->num(iSub); ++iNode) {
	int nd = (*subToNode)[iSub][iNode];
	for(jSub = 0; jSub < nodeToSub->num(nd); ++jSub)
	  if( (subJ=(*nodeToSub)[nd][jSub]) > iSub){ 
	    // we only deal with connection to highered numbered subs

	    if(flag[subJ] != iSub) { 
	      // attribute location for this sub
	      flag[subJ] = iSub;
	      connectedDomain[subJ][nConnect[subJ]] = iSub;
	      connectedDomain[iSub][nConnect[iSub]] = subJ;
	      remoteID[subJ][nConnect[subJ]] = nConnect[iSub];
	      remoteID[iSub][nConnect[iSub]] = nConnect[subJ];
	      whichLocal[subJ] = nConnect[iSub]++;
	      whichRemote[subJ] = nConnect[subJ]++;
	      nodeCount[iSub][whichLocal[subJ]]=1;
	      nodeCount[subJ][whichRemote[subJ]]=1;
	    }  else {
	      nodeCount[iSub][whichLocal[subJ]]++;
	      nodeCount[subJ][whichRemote[subJ]]++;
	    }
	  }
      }
    }
    
    // allocate memory for intertri node lists
    interfNode = new Connectivity *[numSub];

    // interfNode[iSub] is a connectivity for a subdomain to its
    // intertri node (so it is per neighboring subdomain as in:
    // interfNode[3][2][0] is the first node shared by subdomain 3 with
    // its neighbor number two (third neighbor)
    // Similarly, interfNode[3][2]->num() is the number of nodes shared
    // bewteen 3 and 2
    for(iSub=0; iSub < numSub; ++iSub)
      interfNode[iSub] = new Connectivity(nConnect[iSub], nodeCount[iSub]);
    
    // fill the lists
    for(iSub=0; iSub < numSub; ++iSub) {
      flag[iSub] = -1;
      nConnect[iSub] = 0;
    }

    for(iSub=0; iSub < numSub; ++iSub) {

      for(iNode = 0; iNode < subToNode->num(iSub); ++iNode) {
	int nd = (*subToNode)[iSub][iNode];

	for(jSub = 0; jSub < nodeToSub->num(nd); ++jSub)
	  if( (subJ=(*nodeToSub)[nd][jSub]) > iSub){ 
	    // we only deal with connection to highered numbered subs

	    if(flag[subJ] != iSub) { 
	      // attribute location for this sub
	      flag[subJ] = iSub;
	      whichLocal[subJ] = nConnect[iSub]++;
	      whichRemote[subJ] = nConnect[subJ]++;
	      (*interfNode[iSub])[whichLocal[subJ]][0] = nd;
	      (*interfNode[subJ])[whichRemote[subJ]][0] = nd;
	      nodeCount[iSub][whichLocal[subJ]]=1;
	      nodeCount[subJ][whichRemote[subJ]]=1;
	    } else {
	      int il = nodeCount[iSub][whichLocal[subJ]]++;
	      int jl = nodeCount[subJ][whichRemote[subJ]]++;
	      (*interfNode[iSub])[whichLocal[subJ]][il]  = nd;
	      (*interfNode[subJ])[whichRemote[subJ]][jl] = nd;
	    }
	  }
      }  
    }
    // nConnect [iSub] is the number of neighbors	 for sub iSub
    // connectedDomain[iSub][jSub] is the subdomain number that is the
    // jSub'th neighbor to subdomain iSub
    
    // Free allocated memory
    delete [] nodeCount;
    delete [] remoteID;
    delete [] whichLocal;
    delete [] whichRemote;
    delete [] flag;

  } else {
    nConnect = new int[1];
    nConnect[0] = 0;
  }
  
}

//------------------------------------------------------------------------------

void Decomp3D::makeSubD(int noOverlap) {

  if (numSub>1) {
    int* connect;
    int* cx;
    int i,j,k,count;
    
    ElemSet &elems = fluidDom->elems;
    FaceSet &faces = fluidDom->faces;
    
    int numElem = elems.size();
    int numFace = faces.size();
    
    // elemToNode is set locally and used to construct subToNode
    Connectivity* elemToNode = new Connectivity(&elems);
    
    // which is then reversed to get nodeToSub
    subToNode = subToElem->transcon(elemToNode);  
    nodeToSub = subToNode->reverse();
    
    // subToSub is created from subToNode and nodeToSub
    subToSub  = subToNode->transcon(nodeToSub);
    
    // nodeToElem is set locally by reversing elemToNode
    Connectivity* nodeToElem = elemToNode->reverse();
    
    // Allocate connectivity table for faceToElem
    connect=new int[numElem];
    cx=new int[numFace+1];
    count=0;
  
    // Counter for number of nodes per element and flag
    int *visitTag = new int[numElem];
    int *visitCount = new int[numElem];
    
    // Initialize flag
    for(i = 0; i < numElem; ++i)
      visitTag[i] = -1;
    
    // Loop over faces and search it's attached element 
    for (i = 0; i < numFace; i++) {
      int fnodes[Face::MaxNumNd];
      int nnd = faces[i].numNodes();
      
      faces[i].nodes(fnodes);
      cx[i] = count;
      
      // Loop over the attached elements to each node
      // and check when an element contains all of the 
      // nodes of the face.
      bool found = false;
      for (int j = 0; j < nnd; j++) {
	int thisNode = fnodes[j];
	
	// Loop over elements attached to this node
	for (int k = 0; k < nodeToElem->num(thisNode); k++) {
	  
	  int theEle = (*nodeToElem)[thisNode][k];
	  
	  // If we did not visit this element yet
	  // for this face, initialise counter to 0
	  // and tag it to this face
	  if(visitTag[theEle] != i) {
	    visitCount[theEle] = 0;
	    visitTag[theEle] = i;
	  }
	  
	  // Increment number of nodes attached to this element
	  visitCount[theEle]++;
	  
	  // Check if number of nodes attached to this element
	  // is equal to the number of nodes in current face
	  if(visitCount[theEle] == nnd) {
	    connect[count++] = theEle;
	    found = true;
	    
	    // We found the element connected to face i,
	    // stop looping over elements
	    break;
	  }
	}
      }
      if(found == false)
	fprintf(stderr, "Inconsistent mesh: Boundary face is not attached to an element");
    }
    
    cx[numFace]=count;
    
    // Now we can create the faceToElem connectivity
    Connectivity* faceToElem = new Connectivity(numFace, cx, connect);
    Connectivity* elemToFace = faceToElem->reverse();
    subToFace = subToElem->transcon(elemToFace);
    
    // Delete intermediate connectivities
    delete elemToNode;
    delete nodeToElem;
    delete faceToElem;
    delete elemToFace;
    
    // Delete counter and flag arrays
    delete [] visitTag;
    delete [] visitCount;

  } else if(numSub==1){
    int  i, *connect, *cx;
    
    // Create subToNode connectivity
    connect = new int[fluidDom->nodes.size()];
    cx      = new int[2];
    
    for(i=0;i<fluidDom->nodes.size();i++)
      connect[i]=i;
    
    cx[0]=0;
    cx[1]=fluidDom->nodes.size();
    subToNode=new Connectivity(1,cx,connect);
    
    // Create subToFace connectivity
    connect = new int[fluidDom->faces.size()];
    cx      = new int[2];
    
    for(i=0; i<fluidDom->faces.size(); i++)
      connect[i]=i;
    
    cx[0]= 0;
    cx[1]=fluidDom->faces.size();
    subToFace = new Connectivity(1,cx,connect);
    
    // Get nodeToSub by reversing subToNode
    nodeToSub = subToNode->reverse();

    // Get subToSub by combining subToNode and nodeToSub
    subToSub  = subToNode->transcon(nodeToSub);	
  }
  
}

//------------------------------------------------------------------------------

void Decomp3D::print(){
  int i,j,k,ndom;
  int MGN=0,MGE=0,MGF=0,MGI=0;
  char outFile[128];
  
  int* map=new int[fluidDom->nodes.size()];
  // below the flu-* files are created

  // isBound is a boolean array that will contain 1 if
  // a node is on the boundary, 0 otherwise
  int* isBound=new int[fluidDom->nodes.size()];

  fprintf(stderr, "********************************************************\n");
  fprintf(stderr, "WARNING : flu-##### FILE FORMAT HAS CHANGED\n\n");
  fprintf(stderr, "  local node numbers for all elements and faces for each\n");
  fprintf(stderr, "  subdomain are now preceded by the it's number of nodes\n");
  fprintf(stderr, "********************************************************\n");

  for(i=0; i<numSub; i++) {
    sprintf(outFile, "flu-%.5d", i+1);
    FILE* outFilePtr = fopen(outFile, "w");
    
    // a temporary map for the global addresses of the nodes of each
    // subdomain to there local addresses.
    for(j=0;j<subToNode->num(i);j++)
      map[(*subToNode)[i][j]]=j;

    // output the sub number
    fprintf(outFilePtr, "%d\n", i+1);

    // output the number of nodes, elems, and faces in each subs on 1 line
    fprintf(outFilePtr, "%d %d %d\n", 
	    subToNode->num(i), subToElem->num(i), subToFace->num(i));

    // output the number of connected subdomains
    fprintf(outFilePtr, "%d\n", nConnect[i]);

    // output the number of a connected sub and then all points
    // on the intertri in local coords.
    for(j=0; j<nConnect[i]; j++) {
      ndom=connectedDomain[i][j];
      fprintf(outFilePtr, "%d\n", ndom+1);
      fprintf(outFilePtr, "%d\n", (interfNode[i])->num(j));

      // calc the max number of interf nodes on all interf for all subs
      MGI=(MGI<(interfNode[i])->num(j))?(interfNode[i])->num(j):MGI;

      for(k=0;k<(interfNode[i])->num(j);k++)
	fprintf(outFilePtr, "%d\n", map[(*interfNode[i])[j][k]]+1);
    }

    // calc the max number nodes for all subs
    MGN=(MGN<subToNode->num(i))?subToNode->num(i):MGN;

    // output node coords for all nodes in this sub
    for(j=0;j<subToNode->num(i);j++)
      fprintf(outFilePtr, "%e %e %e\n", fluidDom->nodes[(*subToNode)[i][j]][0],
	      fluidDom->nodes[(*subToNode)[i][j]][1], fluidDom->nodes[(*subToNode)[i][j]][2]);
    
    // calc max number of elems for all subs
    MGE=(MGE<subToElem->num(i))?subToElem->num(i):MGE;

    // output local node numbers for all elems in this sub
    for(j=0; j<subToElem->num(i); j++) {
      Elem &el = fluidDom->elems[(*subToElem)[i][j]];

      // Number of nodes per element
      fprintf(outFilePtr, "%d ", el.numNodes());

      // Local node numbers
      for(k=0; k<el.numNodes(); k++)
	fprintf(outFilePtr, " %d", map[el[k]]+1);

      fprintf(outFilePtr, "\n");
    }

    // calc the max number of faces for all subs
    MGF=(MGF<subToFace->num(i))?subToFace->num(i):MGF;

    // output local node numbers for all faces in this sub
    for(j=0;j<subToFace->num(i);j++) {
      Face &fa = fluidDom->faces[(*subToFace)[i][j]];

      // Boundary condition type for this face
      fprintf(outFilePtr, "%d\n", fluidDom->getTypeBC((*subToFace)[i][j]));

      // Number of nodes per element
      fprintf(outFilePtr, "%d ", fa.numNodes());

      // Local node numbers
      for(k=0; k<fa.numNodes(); k++)
	fprintf(outFilePtr, " %d", map[fa[k]]+1);

      fprintf(outFilePtr, "\n");
    }

    for(j=0;j<fluidDom->nodes.size();j++)
      isBound[j]=1;

    for(j=0;j<nConnect[i];j++)
      for(k=0;k<(interfNode[i])->num(j);k++)
       	isBound[(*interfNode[i])[j][k]]=0;

    // output 0 if this node is on the boundary, 1 if not 
    for(j=0; j<subToNode->num(i); j++)
      fprintf(outFilePtr, "%d\n", isBound[(*subToNode)[i][j]]);

    // output a map of local addresses to global addresses for nodes.
    for(j=0; j<subToNode->num(i); j++) {
      fprintf(outFilePtr, "%d\n", (*subToNode)[i][j]+1);
    }

    cerr<<"The file: "<<outFile<<" has been written"<<endl;
    fclose(outFilePtr);
  }	

  // below fluid.glob is created
  ofstream outFilePtr("flu.glob",ios::out);

  // 0's (garbage) for the first three lines, we do not need them
  outFilePtr<<0<<endl;
  outFilePtr<<0<<endl;
  outFilePtr<<0<<endl;

  // num subs
  outFilePtr<<numSub<<endl;

  // num of nodes in domain
  outFilePtr<<fluidDom->nodes.size()<<endl;

  // num of elems in domain
  outFilePtr<<fluidDom->elems.size()<<endl;

  // num faces in domain
  outFilePtr<<fluidDom->faces.size()<<endl; 

  // max number nodes for all subs
  outFilePtr<<MGN<<endl; 
  
  // max number elems for all subs
  outFilePtr<<MGE<<endl; 
  
  // max number faces for all subs
  outFilePtr<<MGF<<endl; 

  // max number of interface nodes for all subs
  outFilePtr<<MGI<<endl; 

  outFilePtr.close();
  cerr<<"The file: flu.glob has been written."<<endl;

}

