#include"ElementClass.h"
#include"ThreeNodeShell.h"
#include"Quadralateral2D.h"
#include"Beam.h"
#include"Quadralateral3D.h"

extern bool beamFlag;
ElementClass*
ElementClass::build(int type,int size, int* node){
	ElementClass* ele;
	if(dimension==2){
		switch(type){
			case THREENODESHELL:
			case THREENODESHELLTHERMAL:
                        case THREENODETRI3D:
				ele=new ThreeNodeShell(size,node);
				break;
                        case FELIPPASHELL:
                        case BTSHELL:
                                if(size == 3)
                                  ele=new ThreeNodeShell(size,node);
                                else
                                  ele=new Quadralateral3D(size,node);
                                break;
			case FOURNODESHELL:
			case FOURNODESHELLTHERMAL:
                        case FOURNODEQUADTHERMAL:
                        case COMPO4NODESHELL:
                        case FOURNODEQUAD3D:
				ele=new Quadralateral3D(size,node);
				break;
			case COMPO3NODESHELL:
				ele=new ThreeNodeShell(size,node);
				break;
			case EULERBEAM:
				if(beamFlag)
                                    ele=new Beam(size,node);
                                else
                                    ele = NULL;
			break;
			case TIMOSHENKOBEAM:
				if(beamFlag)
                                    ele=new Beam(size,node);
                                else
                                    ele = NULL;
			break;
			default:
				ele=NULL;
			break;
		}
	}
	else{
		switch(type){
			case FOURNODEQUAD:
				ele=new Quadralateral2D(size,node);
			break;
			case TWONODETRUSS:
				ele=new Beam(size,node);
			break;
			case EULERBEAM:
				ele=new Beam(size,node);
			break;
			case TIMOSHENKOBEAM:
				ele=new Beam(size,node);
			break;
			default:
				ele=NULL;
			break;
		}
	}
	return ele;
}

ostream& 
operator<<(ostream& os, ElementClass& source){
	int i,num;
	
	num=source.numNodes();
	for(i=0;i<num-1;i++)
		os<<(source.getNode(i)+1)<<' ';
	os<<(source.getNode(num-1)+1);
	return os;
}

void ElementClass::matchNode(ResizeArray<Point3D*>& allPoints, int node, 
                             MatchInfo& info)
{
  // PJSA
  for(int i=0; i<numNodes(); ++i)
    if(node == getNode(i)) {
      info.success = true;
      getLocalCoordinates(i, info.xi, info.eta);
      info.point = *(allPoints[node]);
      return;
    }
}

