#include "FluidDomain.h"

#ifdef OLD_STL
#include <queue.h>
#include <map.h>
#else
#include <queue>
#include <map>
using std::queue;
using std::map;
#endif

#define MAXLINE 500


//--------------------------------------------------------------------

void FluidDomain2D::getGeometry(char* file) {

  int i;

  ifstream fluidFilePtr(file,ios::in);
  if(!fluidFilePtr){
    cerr<<"Error opening file: "<<file;
    cerr<<"DONE"<<endl;
    fluidFilePtr.close();
    exit(1);
  }
  
  //read all mesh info into fluidDom, in 2D there are no
  //elements so hece there number is 0 and the array pointer	
  //where there list would be stored points to NULL
  //also 2D points have 2 coords so the third coord is 0
  
  int numNode, numFace;
  fluidFilePtr >> numNode >> numFace;

  // Resize sets for actual number of faces and nodes
  nodes.resize(numNode);
  faces.resize(numFace);

  // Read in node coordinates
  for(i=0; i<numNode; i++){
    fluidFilePtr >> nodes[i][0] >> nodes[i][1];
    nodes[i][2] = 0.0;
  }

  // Read in face nodes
  for(i=0; i<numFace; i++) {
    faces.addFace(i, new(faces.getMem()) FaceTria());
    fluidFilePtr >> faces[i];
  }

  // Read in boundary condition types
  for(i=0; i<numNode; i++) {
    int t; 
    fluidFilePtr >> t;
    setTypeBC(i, t);
  }

}

//--------------------------------------------------------------------

FluidDomain3D::FluidDomain3D() {

  type_active = new bool[MAX_TYPE_BC - MIN_TYPE_BC + 1];
  type_name = new char*[MAX_TYPE_BC - MIN_TYPE_BC + 1];
  
  type_active -= MIN_TYPE_BC;
  type_name -= MIN_TYPE_BC;
  
  int i;
  for (i=MIN_TYPE_BC; i<=MAX_TYPE_BC; ++i) {
    type_active[i] = false;
    type_name[i] = "InactiveSurface";
  }
  
  type_active[-5] = true;
  type_active[-4] = true;
  type_active[-3] = true;
  type_active[-2] = true;
  type_active[2] = true;
  type_active[3] = true;
  type_active[4] = true;
  type_active[5] = true;
  type_active[6] = true;
  
  type_name[-5] = "OutletMoving";
  type_name[-4] = "InletMoving";
  type_name[-3] = "StickMoving";
  type_name[-2] = "SlipMoving";
  type_name[-1] = "SymmetryMoving";
  type_name[0] = "Internal";
  type_name[1] = "SymmetryFixed";
  type_name[2] = "SlipFixed";
  type_name[3] = "StickFixed";
  type_name[4] = "InletFixed";
  type_name[5] = "OutletFixed";
  type_name[6] = "Symmetry";
  type_name[10] = "Missing";

}

//--------------------------------------------------------------------

FluidDomain3D::~FluidDomain3D() {

  type_active += MIN_TYPE_BC;
  type_name += MIN_TYPE_BC;

  delete [] type_active;
  delete [] type_name;

}

//--------------------------------------------------------------------

void FluidDomain3D::getGeometry(char* file) {

  FILE* fp = fopen(file, "r");
  if (!fp) {
    fprintf(stderr, "Error: could not open file \'%s\'\n", file);
    exit(1);
  }
  char line[MAXLINE], keyword[MAXLINE];
  fgets(line, MAXLINE, fp);
  sscanf(line, "%s", keyword);
  int type = 0;
  if (strcmp(keyword, "Nodes") == 0 || 
      strcmp(keyword, "Elements") == 0 ||
      strcmp(keyword, "typedef") == 0)
    type = 1;
  else if (strcmp(keyword, "FIELDVIEW") == 0)
    type = 2;
  else
    type = 3;
  fclose(fp);

  if (type == 1)
    getFluidDomainXpost(file);
  else if (type == 2)
    getFluidDomainFieldView(file);
  else if (type == 3)
    getFluidDomainSinus(file);

}

//--------------------------------------------------------------------

struct SNode {
  int index;
  double x[3];
};

void FluidDomain3D::getFluidDomainXpost(char* meshname) {

  map<int,int> type_map;
  type_map[4] = 4;
  type_map[5] = 5;

  queue<SNode> node_queue;
  queue<Face*> face_queue;
  queue<Elem*> elem_queue;
  
  FILE* fp = fopen(meshname, "r");
  if (!fp) {
    fprintf(stderr, "Error: could not open file \'%s\'\n", meshname);
    exit(1);
  }

  // To hold current boundary condition 
  // and type of file section 
  int type_boco = 10;
  int type_read = 0;

  // Some counters for statistics
  int quad_count = 0;
  int tria_count = 0;
  int tet_count  = 0;
  int pyr_count  = 0;
  int pri_count  = 0;

  char line[MAXLINE], key1[MAXLINE], key2[MAXLINE];
  bool warnFlag = false;

  while (fgets(line, MAXLINE, fp) != 0) {
    // Read file line per line
    // First figure out what to do with it
    sscanf(line, "%s", key1);
    bool skip = false;
    
    if (strcmp(key1, "Nodes") == 0) {
      // We are about to read a node section
      skip = true;
      type_boco = 10;
      type_read = 1;
      sscanf(line, "%*s %s", key2);

    } else if (strcmp(key1, "Elements") == 0) {
      // We are about to read an element section
      skip = true;
      type_boco = 10;
      type_read = 2;

      sscanf(line, "%*s %s", key2);
      for (int j=MIN_TYPE_BC; j<=MAX_TYPE_BC; ++j) {

	if (strncmp(key2, type_name[j], 
		    strlen(type_name[j])) == 0) {
	  type_boco = j;
	  if (type_boco == -1)  {
            warnFlag = true;
	    type_boco = 6;
          } else if (type_boco == 1)
	    type_boco = 6;
	  break;
	}
      }

    } else if (strcmp(key1, "typedef") == 0) {
      // We are reading a type def section      
      skip = true;
      int t;

      sscanf(line, "%*s %d %s", &t, key2);
      if (strcmp(key2, "Tri_3") == 0)
	type_map[t] = 4;
      else if (strcmp(key2, "Tetra_4") == 0)
	type_map[t] = 5;
    }

    if (!skip) {
      // We are reading some data section:
      // Based on the "type_read" state, fill
      // apropriate data queues

      if (type_read == 1) { 
	// Node : read in coordinates
	SNode sn;
	sscanf(line, "%*s %lf %lf %lf", 
	       &sn.x[0], &sn.x[1], &sn.x[2]);

	// Add node to queue
	node_queue.push(sn);

      } else if (type_read == 2) {
	// We are reading some kind of element (either
	// faces or elements), first apply element type 
	// mapping function if it was defined
	int num, t;

	sscanf(line, "%d %d", &num, &t);
	map<int,int>::iterator it = type_map.find(t);
	if (it != type_map.end())
	  t = it->second;

	// Now read data and add create element 
	// to corresponding queue

	if (t == 2) { 
	  if (type_active[type_boco]) {
	    // Quad Face : read in 4 nodes
	    int nn[4];
	    sscanf(line, "%*s %*s %d %d %d %d", 
		   &nn[0], &nn[1], &nn[2], &nn[3]);
	    for (int k=0; k<4; k++) 
	      nn[k]--;
	    
	    // Create Quad face and add to queue
	    FaceQuad *f = new(faces.getMem()) FaceQuad(nn);
	    f->setTypeBC(type_boco);
	    face_queue.push(f);
	  
	    // Keep track of number of quads
	    quad_count++;
	  }
	} else if (t == 4) { 
	  if (type_active[type_boco]) {
	    // Triangular Face : read in 3 nodes
	    int nn[3];
	    sscanf(line, "%*s %*s %d %d %d", 
		   &nn[0], &nn[1], &nn[2]);
	    for (int k=0; k<3; k++) 
	      nn[k]--;
	    
	    // Create Tria face and add to queue
	    FaceTria *fp = new(faces.getMem()) FaceTria(nn);
	    fp->setTypeBC(type_boco);
	    face_queue.push(fp);
	    
	    // Keep track of number of triangles
	    tria_count++;
	  }
	} else if (t == 5) { 
	  // Tetrahedron Element : read in 4 nodes
	  int nn[4];
	  sscanf(line, "%*s %*s %d %d %d %d", 
		 &nn[0], &nn[1], &nn[2], &nn[3]);
	  for (int k=0; k<4; k++) 
	    nn[k]--;

	  // Create Tet elem and add to queue
	  ElemTet *ep = new(elems.getMem()) ElemTet(nn);
	  elem_queue.push(ep);

	  // Keep track of number of tets
	  tet_count++;

//      } else if (t == ?? ) { // Elem (Pyramid) 
///////   needs to be changed: t = 10 may cause confusion //////
// 	  int nn[5];
// 	  sscanf(line, "%*s %*s %d %d %d %d %d %d", 
// 		 &nn[0], &nn[1], &nn[2], 
// 		 &nn[3], &nn[4]);
// 	  for (int k=0; k<5; k++) 
// 	    nn[k]--;

// 	  ElemPyr *ep = new(elems.getMem()) ElemPyr(nn);
// 	  elem_queue.push(ep);
//      
//	  // Keep track of number of quads
//	  pyr_count++;
//
	} else if (t == 10) { // Elem (Prism)	  
	  // Prismatic Element : read in 4 nodes
	  int nn[6];
	  sscanf(line, "%*s %*s %d %d %d %d %d %d", 
		 &nn[0], &nn[1], &nn[2], 
		 &nn[3], &nn[4], &nn[5]);
	  for (int k=0; k<6; k++) 
	    nn[k]--;

	  // Create Pri elem and add to queue
	  ElemPri *ep = new(elems.getMem()) ElemPri(nn);
	  elem_queue.push(ep);

	  // Keep track of number of quads
	  pri_count++;

	} else {
	  fprintf(stderr, "Error: element %d has a wrong type (%d)\n", num, t);
	  exit(1);
	}	

      } // end if (!skip)

    } else
      fprintf(stdout, "    Found %s (%d)\n", key2, type_boco);

  }

  if (warnFlag)
    fprintf(stderr, " ... WARNING: Symmetry Moving BC converted to Symmetry and will NOT be matched\n"); 

  fclose(fp);

  int numNode = node_queue.size();
  int numFace = face_queue.size();
  int numElem = elem_queue.size();
  int count;

  // Output some statistics
  fprintf(stdout, "    Finished reading mesh\n");
  fprintf(stdout, "      Num of nodes:    %9d\n", numNode);
  fprintf(stdout, "      Num of faces:    %9d (tria %d, quad %d)\n",
	  numFace, tria_count, quad_count);
  fprintf(stdout, "      Num of elements: %9d (tet %d, pri %d, pyr %d)\n",
	  numElem, tet_count, pri_count, pyr_count);

  // Resize sets for actual number of elems
  nodes.resize(numNode);
  faces.resize(numFace);
  elems.resize(numElem);

  // Copy nodes from queue to nodeSet
  count = 0;
  while (node_queue.size()) {
    SNode sn = node_queue.front();
    nodes[count][0] = sn.x[0];
    nodes[count][1] = sn.x[1];
    nodes[count][2] = sn.x[2];
    node_queue.pop();
    count++;
  }

  // Copy faces (pointers) from queue to faceSet
  count = 0;
  while (face_queue.size()) {
    faces.addFace(count, face_queue.front());
    face_queue.pop();
    count++;
  }
  
  // Copy elements (pointers) from queue to elemSet
  count = 0;
  while (elem_queue.size()) {
    elems.addElem(count, elem_queue.front());
    elem_queue.pop();
    count++;
  }

}

//--------------------------------------------------------------------

int cmp_nocase(const char* s, const char* s2, int n)
{

  int size_s = strlen(s);
  int size_s2 = strlen(s2);

  int count = 0;
  while (size_s != 0 && size_s2 != 0) {
    if (tolower(s[count]) != tolower(s2[count]))
      return (tolower(s[count]) < tolower(s2[count])) ? -1 : 1;
    ++count;
    if (n > 0 && count == n)
      return 0;
    --size_s;
    --size_s2;
  }

  size_s = strlen(s);
  size_s2 = strlen(s2);

  return (size_s == size_s2) ? 0 : (size_s < size_s2) ? -1 : 1;

}

//------------------------------------------------------------------------------

void FluidDomain3D::getFluidDomainFieldView(char* meshname)
{

  FILE* fp = fopen(meshname, "r");
  if (!fp) {
    fprintf(stderr, "Error: could not open file \'%s\'\n", meshname);
    exit(1);
  }

  // Read boundary condition names
  char line[MAXLINE], item[MAXLINE];
  while (fgets(line, MAXLINE, fp) != 0) {
    if (cmp_nocase(line, "Boundary Table", 14) == 0)
      break;
  }
  fgets(line, MAXLINE, fp);
  int num_bcs;
  sscanf(line, "%d", &num_bcs);
  int* bcs = new int[num_bcs];
  int i;
  for (i=0; i<num_bcs; ++i) {
    fgets(line, MAXLINE, fp);
    sscanf(line, "%*s %s", item);
    for (int j=MIN_TYPE_BC; j<=MAX_TYPE_BC; ++j) {
      if (cmp_nocase(item, type_name[j], strlen(type_name[j])) == 0) {
	bcs[i] = j;
	if (bcs[i] == -1)
	  bcs[i] = 1;
	//bcs[i] = -2;
	//else if (bcs[i] == 1)
	//bcs[i] = 2;
	break;
      }
    }
    fprintf(stdout, "    Found %s (%d) boundary code\n", item, bcs[i]);
  }

  // Read nodes
  while (fgets(line, MAXLINE, fp) != 0) {
    if (cmp_nocase(line, "Nodes", 5) == 0)
      break;
  }

  fgets(line, MAXLINE, fp);

  int numNode;
  sscanf(line, "%d", &numNode);

  fprintf(stdout, "    Found %d nodes\n", numNode);

  // Resize set for actual number of nodes
  nodes.resize(numNode);
  for (i=0; i<numNode; ++i) {
    fgets(line, MAXLINE, fp);
    sscanf(line, "%lf %lf %lf", &nodes[i][0], &nodes[i][1], &nodes[i][2]);
  }
 
  // Read boundary faces
  while (fgets(line, MAXLINE, fp) != 0) {
    if (cmp_nocase(line, "Boundary Faces", 14) == 0)
      break;
  }
  fgets(line, MAXLINE, fp);

  int numFace;
  int quad_count = 0;
  int tria_count = 0;
  sscanf(line, "%d", &numFace);

  fprintf(stdout, "    Found %d boundary faces\n", numFace);

  // Resize set for actual number of faces
  nodes.resize(numFace);
  for (i=0; i<numFace; ++i) {
    fgets(line, MAXLINE, fp);
    int t, nn;

    sscanf(line, "%d %d", &t, &nn);

    if (nn == 3) {
      faces.addFace(i, new(faces.getMem()) FaceTria());
      sscanf(line, "%*d %*d %d %d %d",
	   &faces[i][0], &faces[i][1], &faces[i][2]);
      tria_count++;

    } else if (nn == 4) {
      faces.addFace(i, new(faces.getMem()) FaceQuad());
      sscanf(line, "%*d %*d %d %d %d %d",
	     &faces[i][0], &faces[i][1], &faces[i][2], &faces[i][3]);
      quad_count++;

    } else {
      fprintf(stderr, "*** Error: wrong number of nodes (%d) for face %d\n", nn, i+1);
      exit(1);
    }

    for (int k=0; faces[i].numNodes(); k++)
      faces[i][k]--;

    faces[i].setTypeBC(bcs[t-1]);
  }

  fprintf(stdout, "    (tria %d, quad %d)\n", tria_count, quad_count);

  fclose(fp);

}

//------------------------------------------------------------------------------

void FluidDomain3D::getFluidDomainSinus(char* name){

  int i;

  ifstream fluidFilePtr(name,ios::in);

  if(!fluidFilePtr){
    cerr<<"Error opening file: "<<name<<endl;
    cerr<<"DONE"<<endl;
    fluidFilePtr.close();
    exit(1);
  }

  int numNode;
  int numElem;
  int numFace;

  // Read number of nodes, elements (tets), and faces (triangles)
  fluidFilePtr >> numNode >> numElem >> numFace;

  
  // Resize sets for actual number of "elements"
  nodes.resize(numNode);
  elems.resize(numElem);
  faces.resize(numFace);

  // Read in node coordinates
  for (i=0; i<numNode; i++)
    fluidFilePtr >> nodes[i];

  // Read in element node numbers (create ElemTet's first)
  for (i=0; i<numElem; i++) {
    elems.addElem(i, new(elems.getMem()) ElemTet());
    fluidFilePtr >> elems[i];
  }

  // Read in face boundary conditions (create FaceTria's first)
  for (i=0; i<numFace; i++) {
    int t;
    fluidFilePtr >> t;
    faces.addFace(i, new(faces.getMem()) FaceTria());
    faces[i].setTypeBC(t);
  }

  for (i=0; i<numFace; i++)
    fluidFilePtr >> faces[i];
  
  fluidFilePtr.close();

  fprintf(stdout, "    Finished reading mesh\n");
  fprintf(stdout, "      Num of nodes:    %9d\n", numNode);
  fprintf(stdout, "      Num of faces:    %9d (tria %d, quad %d)\n",
	  numFace, numFace, 0);
  fprintf(stdout, "      Num of elements: %9d (tet %d, pri %d, pyr %d)\n",
	  numElem, numElem, 0, 0);


}

//------------------------------------------------------------------------------
