#include"ThreeNodeShell.h"
#include <cmath>
#include <limits>


ThreeNodeShell::ThreeNodeShell(int inSize, int* inNodes){
 if (inSize < 3) throw "Not enough nodes";
 nodes[0] = inNodes[0];
 nodes[1] = inNodes[1];
 nodes[2] = inNodes[2];
}
int
ThreeNodeShell::getNode(int position){
	assert(0<=position);
	assert(position<=2);
	return nodes[position];
}

void
ThreeNodeShell::match(ResizeArray<Point3D*>& allPoints,
const Point3D& matchPoint, const double tol, MatchInfo& info)const {

	double s,c,t;
	double A[2][2],b[2];
	double t1;
	int i;
	for(i=0;i<3;i++)
		if (allPoints[nodes[i]]==NULL){
			info.success=0;
			return;
		}
	Point3D p=*(allPoints[nodes[0]]);
	Point3D d1=*(allPoints[nodes[1]])-p;
	Point3D d2=*(allPoints[nodes[2]])-p;
	Point3D mmp=matchPoint-p;
	A[0][0]=dotp(d1,d1);
	A[1][0]=A[0][1]=dotp(d1,d2);
	A[1][1]=dotp(d2,d2);
	b[0]=dotp(d1,mmp);
	b[1]=dotp(d2,mmp);
//To insure stability in the computation we use a Givens rotation
//to solve.  This proceedure is described in Galob and Van Loan's
//book Matrix Computation  from Johns Hopkins University Press 1996.
	//if(fabs(A[1][0])<1.0E-4){
        //PJSA: 1.0E-4 is too big to use as an absolute tolerance, given that the vectors d1 and d2 are not normalized
        if(std::abs(A[1][0]) < std::max(std::numeric_limits<double>::epsilon(),1.0e-4*std::sqrt(A[0][0])*std::sqrt(A[1][1]))){
		c=1.0;
		s=0.0;
	}
	else {
		if(fabs(A[1][0])>fabs(A[0][0])){
			t=-A[0][0]/A[1][0];
			s=1.0/(sqrt(1.0+t*t));
			c=s*t;
		}
		
		else{
			t=-A[1][0]/A[0][0];
			c=1.0/(sqrt(1.0+t*t));
			s=c*t;
		}
	}
	t1=A[0][1];
	A[0][0]=c*A[0][0]-s*A[1][0];
	A[0][1]=c*A[0][1]-s*A[1][1];
	A[1][1]=s*t1+c*A[1][1];
	t1=b[0];
	b[0]=c*b[0]-s*b[1];
	b[1]=s*t1+c*b[1];
	info.eta=b[1]/A[1][1];
	info.xi=(b[0]-info.eta*A[0][1])/A[0][0];
	info.success=0;
	if((-tol<=info.xi)&&(-tol<=info.eta)&&((info.eta+info.xi)<(1.0+2.0*tol))){
		Point3D cp=crossp(d1,d2);
		Point3D n=cp/norm(cp);
		double alpha=dotp(n,mmp);
		info.point=matchPoint-alpha*n;
		info.success=1;
		int flag=0;
		if(info.xi<0.0) flag+=1;
		if(info.eta<0.0) flag+=2;
		if(info.eta+info.xi>1.0) flag+=4;
		switch(flag){
			case 0:
				info.dist=fabs(alpha);
			break;
			case 1:
				cp=crossp(d2,n);
				n=cp/norm(cp);
				info.dist=fabs(alpha)+fabs(dotp(info.point-*(allPoints[nodes[0]]),n));
			break;
			case 2:
				cp=crossp(d1,n);
				n=cp/norm(cp);
				info.dist=fabs(alpha)+fabs(dotp(info.point-*(allPoints[nodes[0]]),n));	
			break;
			case 3:
				info.dist=fabs(alpha)+norm(info.point-*(allPoints[nodes[0]]));
			break;
			case 4:
				cp=crossp(d2-d1,n);
				n=cp/norm(cp);
				info.dist=fabs(alpha)+fabs(dotp(info.point-*(allPoints[nodes[1]]),n));
			break;
			case 5:
				info.dist=fabs(alpha)+norm(info.point-*(allPoints[nodes[2]]));
			break;
			case 6:
				info.dist=fabs(alpha)+norm(info.point-*(allPoints[nodes[1]]));
			break;
		}
	}
}
	
void
ThreeNodeShell::getLocalCoordinates(int i, double &xi, double &eta)
{
  switch(i) {
    case 0:
      eta = 0.0;
      xi = 0.0;
    break;
    case 1:
      xi = 1.0;
      eta = 0.0;
    break;
    case 2:
      xi = 0.0;
      eta = 1.0;
    break;
  }
}
