#include "aux.h"
#include "values.h"


extern int dimension;
NFrameData null_nframe;
//dimension is a global variable that stores the problems dimension.
//It is defined in the main program.
Domain* domain=new Domain;

ElementClass* 
buildElem(int type, int size, int* nodes){
	ElementClass* ele;
	ele=ElementClass::build(type,size,nodes);
	return ele;
}

Domain::Domain()
 : allNodes(NULL),allElem(NULL),nfd(null_nframe)
{
  numElem=0;
  maxNode=0;
  maxElem=0;
  numNframes=0;
  basicPosCoords=true;
}

void
Domain::readAux(char* auxFile){
	char buffer[20];
	int num=0,exEl=0,nset=0,eset=0;
	int i,k;
	int flag1,flag2;
	int* excluded=new int[max(maxElem,maxNode)];
	ifstream auxPtr(auxFile,ios::in);
	if(!auxPtr){
		cerr<<"Error opening file: "<<auxFile<<endl;
		cerr<<"DONE"<<endl;
		auxPtr.close();
		exit(1);
	}
	for(k=0;k<2;k++){
		auxPtr>>buffer;
		if((strncmp(buffer,"EIFS",4)==0)&&(eset==0)){
			eset=1;
			auxPtr>>num;
			if(num>0){
				flag1=1;
				flag2=0;
			}
			else{
				flag1=0;
				flag2=1;
			}
			num=abs(num);
			for(i=0;i<maxElem;i++)
				excluded[i]=flag1;
			for(i=0;i<num;i++){
				auxPtr>>exEl;
				excluded[exEl-1]=flag2;
			}
			for(i=0;i<maxNode;i++)
				if(excluded[i]==1)
					allElem[i]=NULL;
		}
		else if((strncmp(buffer,"NIFS",4)==0)&&(nset==0)){
			nset=1;
			auxPtr>>num;
			if(num>0){
				flag1=1;
				flag2=0;
			}
			else{
				flag1=0;
				flag2=1;
			}
			num=abs(num);
			for(i=0;i<maxNode;i++)
				excluded[i]=flag1;
			for(i=0;i<num;i++){
				auxPtr>>exEl;
				excluded[exEl-1]=flag2;
			}
			for(i=0;i<maxNode;i++)
				if(excluded[i]==1)
					allNodes[i]=NULL;
		}
		else if (eset==1 || nset==1)
			break;
		else{
			cerr<<"Bad format in Auxillary file: "<<auxFile<<endl;
			cerr<<"at :"<<buffer<<endl;
			cerr<<"DONE"<<endl;
			exit(1);
		}
	}
	auxPtr.close();
}

void
Domain::getDecomp(char* name){
	char buffer[80];
	int i,j,numElem,num;
	belongsTo=new int[maxElem];
	if(name==NULL){
		for(i=0;i<maxElem;i++)
			belongsTo[i]=0;
		numSub=1;
		return;
	}
	else{
		ifstream in(name,ios::in);
  	if(!in){
    	cerr<<"Error opening file: "<<name<<endl;
			cerr<<"DONE"<<endl;
    	in.close();
    	exit(1);
  	}
	  in.get(buffer[0]);
    i=0;
    while(buffer[i]!='\n'){
      i++;
      in.get(buffer[i]);
    }
    while (strncmp(buffer,"Decomposition",13)){
      in.get(buffer[0]);
      i=0;
      while(buffer[i]!='\n'){
        i++;
        in.get(buffer[i]);
      }
    }
    in>>numSub;
    if(99999<numSub||numSub<0){
      cerr<<"*** ERROR:The number of subdomains must be positive and can't ";
      cerr<<"exceed 99999."<<endl;
			cerr<<"DONE"<<endl;
      exit(1);
    }
		for(i=0;i<numSub;i++){
			in>>numElem;
			for(j=0;j<numElem;j++){
				in>>num;
				belongsTo[num-1]=i;	
			}
		}
	}
}
		
void
Domain::addElem(int nnum, ElementClass* Elem){
	numElem++;

	if(maxElem<=nnum)
		maxElem=nnum+1;
	allElem[nnum]=Elem;
}

void
Domain::addNode(int nnum, double xyz[3], int cp){
  allNodes[nnum] = new Point3D(xyz[0],xyz[1],xyz[2],cp);
  if(nnum >= maxNode) maxNode = nnum+1;
}

void
Domain::matchPoint(Point3D& p, double tol, interCoords& coords, double maxdist){
	double dist;
	int i;
	MatchInfo info;
	dist = MAXDOUBLE;
	coords.success=0;
	info.success=0;
	for(i=0;i<maxElem;i++){
		if(allElem[i]!=NULL){
			allElem[i]->match(allNodes, p,tol,info);
                        // if((info.success==1)&&(dist>info.dist)) {
			if((info.success==1)&&(dist>info.dist)&&(info.dist<=maxdist)){
				dist=info.dist;
				coords.success=1;
				coords.elNum=i;
				coords.xi=info.xi;
				coords.eta=info.eta;
				coords.point=info.point;
				coords.domNum=belongsTo[i];
			}
		}
	}
}

void
Domain::matchPoint(int i, Point3D& p, double tol, interCoords& coords)
{
  // PJSA: search for match only with specified element i
  MatchInfo info;
  coords.success=0;
  info.success=0;
  if(allElem[i]!=NULL) {
    allElem[i]->match(allNodes, p,tol,info);
    if(info.success==1) {
      coords.success=1;
      coords.elNum=i;
      coords.xi=info.xi;
      coords.eta=info.eta;
      coords.point=info.point;
      coords.domNum=belongsTo[i];
    }
  }
}

void
Domain::matchPoint(int node, interCoords& coords)
{
  // PJSA: find first elements containing node and enforce match
  double dist;
  int i;
  MatchInfo info;
  coords.success=0;
  info.success=0;
  for(i=0;i<maxElem;i++) {
    if(allElem[i]!=NULL) {
      allElem[i]->matchNode(allNodes, node, info);
      if(info.success==1) {
        coords.success=1;
        coords.elNum=i;
        coords.xi=info.xi;
        coords.eta=info.eta;
        coords.point=info.point;
        coords.domNum=belongsTo[i];
        return; // just take the first element that contains the node
      }
    }
  }
}


void
Domain::printAllNodes() {
 cout << "List of all nodes"<<endl;
 for(int i = 0; i < maxNode; ++i)
   if(allNodes[i] != NULL)
     cout << i+1 << " " << *(allNodes[i]) << endl;
}

void
Domain::printAllElem(){
	cout<<"List of all elements"<<endl;
	for(int i = 0; i < maxElem; ++i)
		 if(allElem[i] != 0)
				cout << i+1 << " " << *(allElem[i]) << endl;
}
extern int yyparse();

int Domain::setNodalFrame(int id, double *origin, double *data, int type)
{
  int i,j;
  for(i = 0; i < 3; ++i) {
   nfd[id].origin[i] = origin[i];
   for(j = 0; j < 3; ++j)
     nfd[id].frame[i][j] = data[i*3+j];
  }
  nfd[id].type = (NFrameData::FrameType) type;

  return 0;
}

void Domain::transformCoords()
{
  // transform node coordinates from POS_FRM to basic coordinates
  // see: http://en.wikipedia.org/wiki/List_of_canonical_coordinate_transformations#3-Dimensional
  using std::cos;
  using std::sin;
  for(int i = 0; i < maxNode; ++i) {
    if(allNodes[i] == NULL) continue;
    int cp = allNodes[i]->getPosFrm();
    if(cp == 0) continue;

    double v[3];
    switch(nfd[cp].type) {
      default:
      case NFrameData::Rectangular: {
        v[0] = (*allNodes[i])[0]; 
        v[1] = (*allNodes[i])[1];
        v[2] = (*allNodes[i])[2];
      } break;
      case NFrameData::Cylindrical: {
        double rho   = (*allNodes[i])[0];
        double theta = (*allNodes[i])[1];
        v[0] = rho*cos(theta);
        v[1] = rho*sin(theta);
        v[2] = (*allNodes[i])[2];
        break;
      }
      case NFrameData::Spherical: {
        double rho   = (*allNodes[i])[0];
        double theta = (*allNodes[i])[1];
        double phi   = (*allNodes[i])[2];
        v[0] = rho*sin(theta)*cos(phi);
        v[1] = rho*sin(theta)*sin(phi);
        v[2] = rho*cos(theta);
        break;
      }
    }

    // x = T^t*v + o
    for(int j=0; j<3; ++j) 
      (*allNodes[i])[j] = nfd[cp].frame[0][j]*v[0]+nfd[cp].frame[1][j]*v[1]+nfd[cp].frame[2][j]*v[2]
                          + nfd[cp].origin[j];
  }
}

void
buildDomain(){
  int error = yyparse();

  if(error) {
    fprintf(stderr,"Input file contained errors. Aborting\n");
    cerr<<"DONE"<<endl;
    exit(error);
  }

  if(!domain->basicPosCoords) {
    fprintf(stderr, "... Processing nodal frames\n");
    domain->transformCoords();
  }
}

